import config
import app


class Guess:

    def GET(self):
        if app.session.loggedin is True:
            username = app.session.username
            privilege = app.session.privilege
            params = {}
            params['username'] = username
            params['privilege'] = privilege
            return config.render.guess(params)
        else:
            raise config.web.seeother('/login')



from . import config
import hashlib
import app


class Insert():

    def __init__(self):
        pass

    def GET(self):
        if app.session.loggedin is True:
            #username = app.session.username
            privilege = app.session.privilege
            if  privilege == 0:
                return self.GET_INSERT()
            elif privilege == 1:
                raise config.web.seeother('/guess')
        else:
            raise config.web.seeother('/login')

    def POST(self):
        if app.session.loggedin is True:
            #username = app.session.username
            privilege = app.session.privilege
            if  privilege == 0:
                return self.POST_INSERT()
            elif privilege == 1:
                raise config.web.seeother('/guess')
        else:
            raise config.web.seeother('/login')

    def GET_INSERT(self):
        return config.render.insert()

    def POST_INSERT(self):
        form = config.web.input()
        pwdhash = hashlib.md5(form.password).hexdigest()

        config.model.insert_users(
            form['username'],
            pwdhash,
            form['privilege']
        )
        raise config.web.seeother('/users')



import web
from . import config

db = config.db


def validate_user(username, password):
    try:
        return db.select('users',
        where='username=$username and password=$password', vars=locals())[0]
    except Exception as e:
        print(("Model get all Error {}".format(e.args)))
        print(("Model get all Message {}".format(e.message)))
        return None


def get_all_users():
    try:
        return db.select('users')
    except Exception as e:
        print(("Model get all Error {}".format(e.args)))
        print(("Model get all Message {}".format(e.message)))
        return None


def get_users(username):
    try:
        return db.select('users', where='username=$username', vars=locals())[0]
    except Exception as e:
        print(("Model get Error {}".format(e.args)))
        print(("Model get Message {}".format(e.message)))
        return None


def delete_users(username):
    try:
        return db.delete('users', where='username=$username', vars=locals())
    except Exception as e:
        print(("Model delete Error {}".format(e.args)))
        print(("Model delete Message {}".format(e.message)))
        return False


def insert_users(username, password, privilege):
    try:
        db.insert('users',
        username=username,
        password=password,
        privilege=privilege)
    except Exception as e:
        print(("Model insert Error {}".format(e.args)))
        print(("Model insert Message {}".format(e.message)))
        return None


def edit_users(username, password, privilege):
    try:
        db.update('users', username=username,
password=password,
privilege=privilege,
                  where='username=$username',
                  vars=locals())
    except Exception as e:
        print(("Model update Error {}".format(e.args)))
        print(("Model updateMessage {}".format(e.message)))
        return None



import web
import config
import json

class Api_table_name:
    def get(self, primary_key):
        try:
            # http://0.0.0.0:8080/api_table_name?user_hash=12345&action=get
            if primary_key == None:
                result = config.model.get_all_table_name()
                table_name_json = []
                for row in result:
                    tmp = dict(row)
                    table_name_json.append(tmp)
                web.header('Content-Type', 'application/json')
                return json.dumps(table_name_json)
            else:
                # http://http://0.0.0.0:8080/api_table_name?user_hash=12345&action=get&primary_key=1
                result = config.model.get_table_name(int(primary_key))
                table_name_json = []
                table_name_json.append(dict(result))
                web.header('Content-Type', 'application/json')
                return json.dumps(table_name_json)
        except Exception as e:
            print "GET Error {}".format(e.args)
            table_name_json = '[]'
            web.header('Content-Type', 'application/json')
            return json.dumps(table_name_json)

# https://0.0.0.0:8080/api_table_name?user_hash=12345&action=put&primary_key=1&product=nuevo&description=nueva&stock=10&purchase_price=1&price_sale=3&product_image=0
    def put(self, fields):
        try:
            config.model.insert_table_name(fields)
            table_name_json = '[{200}]'
            web.header('Content-Type', 'application/json')
            return json.dumps(table_name_json)
        except Exception as e:
            print "PUT Error {}".format(e.args)
            return None

# http://0.0.0.0:8080/api_table_name?user_hash=12345&action=get&primary_key=1
    def delete(self, primary_key):
        try:
            config.model.delete_table_name(primary_key)
            table_name_json = '[{200}]'
            web.header('Content-Type', 'application/json')
            return json.dumps(table_name_json)
        except Exception as e:
            print "DELETE Error {}".format(e.args)
            return None

# http://0.0.0.0:8080/api_table_name?user_hash=12345&action=update&primary_key=1&product=nuevo&description=nueva&stock=10&purchase_price=1&price_sale=3&product_image=default.jpg
    def update(self, primary_key, fields):
        try:
            config.model.edit_table_name(field_all)
            table_name_json = '[{200}]'
            web.header('Content-Type', 'application/json')
            return json.dumps(table_name_json)
        except Exception as e:
            print "GET Error {}".format(e.args)
            table_name_json = '[]'
            web.header('Content-Type', 'application/json')
            return json.dumps(table_name_json)

    def GET(self):
        user_data = web.input(
            user_hash=None,
            action=None,
field_none
        )
        try:
            user_hash = user_data.user_hash  # user validation
            action = user_data.action  # action GET, PUT, DELETE, UPDATE
field_user_data

            if user_hash == '12345':  # user_hash
                if action == None:
                    raise web.seeother('/404')
                elif action == 'get':
                    return self.get(primary_key)
                elif action == 'put':
                    return self.put(fields)
                elif action == 'delete':
                    return self.delete(primary_key)
                elif action == 'update':
                    return self.update(primary_key, fields)
            else:
                raise web.seeother('/404')
        except Exception as e:
            print "WEBSERVICE Error {}".format(e.args)
            raise web.seeother('/404')



from . import config
import app
import hashlib


class Login:
    
    def __init__(self):
        pass

    def GET(self, *a):
        message = None
        return config.render.login(message)

    def POST(self, *a):
        i = config.web.input()
        pwdhash = hashlib.md5(i.password).hexdigest()
        check = config.model.validate_user(i.username, pwdhash)
        if check:
            app.session.loggedin = True
            app.session.username = check['username']
            app.session.privilege = check['privilege']
            raise config.web.seeother('/')
        else:
            message = "User or Password are not correct!!!!"
            return config.render.login(message)


import config

class Insert():

    def __init__(self):
        pass

    def GET(self):
        return config.render.insert()

    def POST(self):
        form = config.web.input()

        config.model.insert_table_name(
            fields
        )
        raise config.web.seeother('/table_name')



from . import config
import app


class Index:

    def GET(self):
        if app.session.loggedin is True:
            username = app.session.username
            privilege = app.session.privilege
            if  privilege == 0:
                params = {}
                params['username'] = username
                params['privilege'] = privilege
                #return config.render.index(username)
                return config.render.index(params)
            elif privilege == 1:
                raise config.web.seeother('/guess')
        else:
            raise config.web.seeother('/login')



from . import config
import app


class Index:

    def GET(self):
        if app.session.loggedin is True:
            #username = app.session.username
            privilege = app.session.privilege
            if  privilege == 0:
                return self.GET_INDEX()
            elif privilege == 1:
                raise config.web.seeother('/guess')
        else:
            raise config.web.seeother('/login')

    def GET_INDEX(self):
        result = config.model.get_all_users()
        return config.render.index(result)



import web
import os

class View_generator:
    
    db_host = ''
    db_name = ''
    db_user = ''
    db_pw = ''

    table_name = ''

    db = web.database(
            dbn='mysql',
            host=db_host,
            db=db_name,
            user=db_user,
            pw=db_pw
            )

    def conectar(self):
        try:
            new_db = web.database(
            dbn='mysql',
            host=self.db_host,
            db=self.db_name,
            user=self.db_user,
            pw=self.db_pw
            )
            self.db = new_db
            print 'Conectado'
        except Exception as e:
            print e.args
            print "Error en conexion {}".format(e.message)

    def get_tables(self):
        try:
            return self.db.query('show tables')
        except Exception as e:
            print e.args
            print "Error en get_tables {}".format(e.message)
            return None

    def describre_tables(self,table):
        try:
            sql = 'describe {}'.format(table)
            return self.db.query(sql, vars=locals())
        except Exception as e:
            print e.args
            print "Error en describre_tables {}".format(e.message)
            return None

    def get_fields(self,table):
        try:
            result = self.describre_tables(table)
            fields = []
            for i in result:
                r = [i.Field, i.Key]
                fields.append(r)
            return fields
        except Exception as e:
            print e.args
            print "Error en get_fields {}".format(e.message)
            return None

    def get_primary_key(self,fields):
        primary_key = ''
        for field in fields:
            try:
                if field[1] == 'PRI':
                    primary_key = field[0]
                    exit
            except:
                primary_key = None
        return primary_key

    def super_cool(self,table_name):
        try:
            primary_key = self.get_primary_key(self.get_fields(table_name))

            fields = ""
            for field in self.get_fields(table_name)[1:]:
                fields += field[0] + ','
            fields = fields[:-1]

            first_field = fields[0] 


            values = ""
            for field in self.get_fields(table_name)[1:]:
                values += field[0] + '=' + field[0] + ',\n'
            values = values[:-2]

            
            field_all = ""
            for field in self.get_fields(table_name):
                field_all += field[0] + ','
            field_all = field_all[:-1]

            value_all = ""
            for field in self.get_fields(table_name):
                value_all += field[0] + '=' + field[0] + ',\n'
            value_all = value_all[:-2]

            script_path = os.path.abspath(__file__) 
            script_dir = os.path.split(script_path)[0]
            rel_path = "views"
            new_file_path = os.path.join(script_dir,table_name)
            abs_file_path = os.path.join(script_dir, rel_path)

            os.mkdir(os.path.expanduser(table_name))

            file = open(table_name + '/__init__.py', 'w')
            file.write("")
            file.close()

            #Master.html
            
            master_file = open(abs_file_path+'/master.html','r')
            texto =''
            for f in master_file:
                texto += f
            
            texto = texto.replace('table_name', table_name)

            file = open(table_name + '/master.html', 'w')
            file.write(texto)
            file.close()

            #index.html
            
            index_file = open(abs_file_path+'/index.html','r')
            plantilla = ''
            for line in index_file:
                plantilla += line

            plantilla = plantilla.replace('table_name', table_name)
            plantilla = plantilla.replace('primary_key', primary_key)
            plantilla = plantilla.replace('first_field', first_field)
            plantilla = plantilla.replace('fields', fields)
            plantilla = plantilla.replace('values', values)
            plantilla = plantilla.replace('field_all', field_all)
            plantilla = plantilla.replace('value_all', value_all)

            view_index = plantilla
            
            file = open(table_name + '/index.html', 'w')
            file.write(view_index)
            file.close()
            
            #insert.html
            
            insert_file = open(abs_file_path+'/insert.html','r')
            field_file = open(abs_file_path+'/field_insert.md','r')
            texto = ''
            for t in field_file:
                texto += t

            plantilla = ''
            for field in self.get_fields(table_name)[1:]:
                for line in texto:
                    plantilla += line 
                plantilla = plantilla.replace('field', field[0])

            insert = ''
            for t in insert_file:
                insert += t
            
            insert = insert.replace('fields', plantilla)

            
            file = open(table_name + '/insert.html', 'w')
            file.write(insert)
            file.close()
            

            #edit.html
            
            edit_file = open(abs_file_path+'/edit.html','r')
            field_file = open(abs_file_path+'/field_edit.md','r')
            texto = ''
            for t in field_file:
                texto += t

            fields = ''
            for field in self.get_fields(table_name)[1:]:
                for line in texto:
                    fields += line 
                fields = fields.replace('field', field[0])

            edit = ''
            for t in edit_file:
                edit += t
            
            edit = edit.replace('fields', fields)
            edit = edit.replace('table_name', table_name)
            edit = edit.replace('primary_key', primary_key)

            
            file = open(table_name + '/edit.html', 'w')
            file.write(edit)
            file.close()
            
            #delete.html
            
            delete_file = open(abs_file_path+'/delete.html','r')
            field_file = open(abs_file_path+'/field_edit.md','r')
            texto = ''
            for t in field_file:
                texto += t

            fields = ''
            for field in self.get_fields(table_name)[1:]:
                for line in texto:
                    fields += line 
                fields = fields.replace('field', field[0])

            delete  = ''
            for t in delete_file:
                delete += t
            
            delete = delete.replace('fields', fields)
            delete = delete.replace('table_name', table_name)
            delete = delete.replace('primary_key', primary_key)

            
            file = open(table_name + '/delete.html', 'w')
            file.write(delete)
            file.close()
            

            #view.html
            
            view_file = open(abs_file_path+'/view.html','r')
            field_file = open(abs_file_path+'/field_view.md','r')
            texto = ''
            for t in field_file:
                texto += t

            fields = ''
            for field in self.get_fields(table_name)[1:]:
                for line in texto:
                    fields += line 
                fields = fields.replace('field', field[0])

            view  = ''
            for t in view_file:
                view += t
            
            view = view.replace('fields', fields)
            view = view.replace('table_name', table_name)

            file = open(table_name + '/view.html', 'w')
            file.write(view)
            file.close()
            

        except Exception as e:
            print e.args
            print "Error en supercool {}".format(e.message)
            print "Error en supercool {}".format(e)

    def generate(self):
        try:
            for i in self.get_tables():
                m = str(i.values())
                self.super_cool(m[3:-2])
        except Exception as e:
            print "Error en generate {}".format(e.message)

    def generate_one(self, table_name):
        try:
            self.super_cool(table_name)
        except Exception as e:
            print "Error al generar {}".format(e.message)



import config

class Delete:

    def GET(self, primary_key):
        result = config.model.get_table_name(int(primary_key))
        return config.render.delete(result)

    def POST(self, primary_key):
        form = config.web.input()
        config.model.delete_table_name(form['primary_key'])
        raise config.web.seeother('/table_name')



from . import config
import app


class Delete:

    def GET(self, username, **k):
        if app.session.loggedin is True:
            #username = app.session.username
            privilege = app.session.privilege
            if  privilege == 0:
                return self.GET_DELETE(username)
            elif privilege == 1:
                raise config.web.seeother('/guess')
        else:
            raise config.web.seeother('/login')

    def POST(self, username, **k):
        if app.session.loggedin is True:
            #username = app.session.username
            privilege = app.session.privilege
            if  privilege == 0:
                return self.POST_DELETE(username)
            elif privilege == 1:
                raise config.web.seeother('/guess')
        else:
            raise config.web.seeother('/login')

    def GET_DELETE(self, username, **k):
        message = None
        result = config.model.get_users(username)
        return config.render.delete(result, message)

    def POST_DELETE(self, username, **k):
        form = config.web.input()
        e = config.model.delete_users(form['username'])
        if e is False:
            message = "Error al eliminar"
            result = config.model.get_users(form['username'])
            return config.render.delete(result, message)
        else:
            raise config.web.seeother('/users')



import shutil, errno
import sys
from subprocess import call
import os
from model_generator import Model_generator as model_generator
from api_generator import Api_generator as api_generator
from view_generator import View_generator as view_generator
from controller_generator import Controller_generator as controller_generator


version = '0.6.4.4'

def new_base(args):
    name = args[2]
    print 'Creating new project ....'
    print 'Project: {}'.format(name)
    script_path = os.path.abspath(__file__)
    script_dir = os.path.split(script_path)[0]
    rel_path = "mvc_base"
    abs_file_path = os.path.join(script_dir, rel_path)
    #call(['ls -l'], shell = True)
    copyanything(abs_file_path, name)
    print 'Project created succesful'

def new_login(args):
    name = args[2]
    print 'Creating new login project ....'
    print 'Project: {}'.format(name)
    script_path = os.path.abspath(__file__)
    script_dir = os.path.split(script_path)[0]
    rel_path = "mvc_sessions"
    abs_file_path = os.path.join(script_dir, rel_path)
    #call(['ls -l'], shell = True)
    copyanything(abs_file_path, name)
    print 'Project created succesful'

def new_forms(args):
    name = args[2]
    print 'Creating new project ....'
    print 'Project: {}'.format(name)
    script_path = os.path.abspath(__file__)
    script_dir = os.path.split(script_path)[0]
    rel_path = "mvc_forms"
    abs_file_path = os.path.join(script_dir, rel_path)
    #call(['ls -l'], shell = True)
    copyanything(abs_file_path, name)
    print 'Project created succesful'
def copyanything(src, dst):
    try:
        shutil.copytree(src, dst)
    except OSError as exc:
        if exc.errno == errno.ENOTDIR:
            shutil.copy(src, dst)
        else: raise

def new(args):
    name = args[2]
    print 'Creating new project ....'
    print 'Project: {}'.format(name)
    script_path = os.path.abspath(__file__)
    script_dir = os.path.split(script_path)[0]
    rel_path = "mvc"
    abs_file_path = os.path.join(script_dir, rel_path)
    #call(['ls -l'], shell = True)
    copyanything(abs_file_path, name)
    print 'Project created succesful'

def info():
    print 'kuorra V {}'.format(version)
    print 'Author	: Salvador Hernandez Mendoza'
    print 'Email 	: salvadorhm@gmail.com'
    print 'Twiter 	: @salvadorhm'

def help():
    print 'kuorra V {}'.format(version)
    print ''
    print 'HELP'
    print ''
    print 'project_name \t- Project name'
    print 'host \t- Database host'
    print 'db \t- Database name'
    print 'user \t- Database user'
    print 'pw \t- Database password'
    print 'table \t- Database table name'
    print ''
    print 'COMMANDS'
    print ''
    print 'kuorra -n project_name -> NEW PROJECT'
    print 'kuorra -nb project_name -> NEW BLANK PROJECT'
    print 'kuorra -nf project_name -> NEW WEB.PY FORMS USE'
    print 'kuorra -nl project_name -> NEW WEB.PY LOGIN PROJECT'
    print 'kuorra -d -> DEPLOY PROJECT'
    print 'kuorra -i -> INFORMATION'
    print 'kuorra -am db host user pw -> CREATE ALL TABLES MODELS'
    print 'kuorra -m db table host user pw -> CREATE ONE TABLE MODEL'
    print 'kuorra -aa db host user pw -> CREATE ALL TABLES API'
    print 'kuorra -a db table host user pw -> CREATE ONE TABLE API'
    print 'kuorra -av db host user pw -> CREATE ALL TABLES VIEWS'
    print 'kuorra -v db table host user pw -> CREATE ONE TABLE VIEW'
    print 'kuorra -ac db host user pw -> CREATE ALL TABLES CONROLLERS'
    print 'kuorra -c db table host user pw -> CREATE ONE TABLE CONTROLLER'
    print ''
    

def deploy():
    try:
        call(['clear'], shell = False)
        print '*******************************************************'
        print '*kuorra V {}'.format(version)
        print '*  kuorra WebApp Deploy'
        print '*  For OPEN use Crtl + click in URL'
        print '*  For STOP use Crtl + C  '
        print '*******************************************************'
        print '*************    kuorra Console       *****************'
        print '*******************************************************\n'
        call(['python app.py'], shell = True)
    except (KeyboardInterrupt, SystemExit):
        print 'kuorra shutdown..'

def all_models(args):
    name = args[2]
    db_host = args[3]
    db_name = name
    db_user = args[4]
    db_pw = args[5]

    model = model_generator()

    model.db_host = db_host
    model.db_name = db_name
    model.db_user = db_user
    model.db_pw = db_pw

    model.table_name = name

    model.conectar()
    model.generate()

def one_model(args):
    name = args[2]
    db_table = args[3]
    db_host = args[4]
    db_name = name
    db_user = args[5]
    db_pw = args[6]

    model = model_generator()


    model.db_host = db_host
    model.db_name = db_name
    model.db_user = db_user
    model.db_pw = db_pw
    model.table_name = db_table

    model.conectar()
    model.generate_one(db_table)

def all_apis(args):
    name = args[2]
    db_host = args[3]
    db_name = name
    db_user = args[4]
    db_pw = args[5]

    api = api_generator()

    api.db_host = db_host
    api.db_name = db_name
    api.db_user = db_user
    api.db_pw = db_pw

    api.table_name = name

    api.conectar()
    api.generate()

def one_api(args):
    name = args[2]
    db_table = args[3]
    db_host = args[4]
    db_name = name
    db_user = args[5]
    db_pw = args[6]

    api = api_generator()


    api.db_host = db_host
    api.db_name = db_name
    api.db_user = db_user
    api.db_pw = db_pw
    api.table_name = db_table
    
    api.conectar()
    api.generate_one(db_table)

def all_views(args):
    name = args[2]
    db_host = args[3]
    db_name = name
    db_user = args[4]
    db_pw = args[5]

    view = view_generator()

    view.db_host = db_host
    view.db_name = db_name
    view.db_user = db_user
    view.db_pw = db_pw

    view.table_name = name

    view.conectar()
    view.generate()

def one_view(args):
    name = args[2]
    db_table = args[3]
    db_host = args[4]
    db_name = name
    db_user = args[5]
    db_pw = args[6]

    view = view_generator()


    view.db_host = db_host
    view.db_name = db_name
    view.db_user = db_user
    view.db_pw = db_pw
    view.table_name = db_table
   
    view.conectar()
    view.generate_one(db_table)

def all_controllers(args):
    name = args[2]
    db_host = args[3]
    db_name = name
    db_user = args[4]
    db_pw = args[5]

    controller = controller_generator()

    controller.db_host = db_host
    controller.db_name = db_name
    controller.db_user = db_user
    controller.db_pw = db_pw


    controller.table_name = name

    controller.conectar()
    controller.generate()

def one_controller(args):
    name = args[2]
    db_table = args[3]
    db_host = args[4]
    db_name = name
    db_user = args[5]
    db_pw = args[6]

    controller = controller_generator()

    controller.db_host = db_host
    controller.db_name = db_name
    controller.db_user = db_user
    controller.db_pw = db_pw
    controller.table_name = db_table

    controller.conectar()
    controller.generate_one(db_table)

def main():
    try:
        command = sys.argv[1]
        if command == 'new' or command == '-n':
            new(sys.argv)
        elif command == 'new_base' or command == '-nb':
            new_base(sys.argv)
        elif command == 'new_forms' or command == '-nf':
            new_forms(sys.argv)
        elif command == 'new_login' or command == '-nl':
            new_login(sys.argv)
        elif command == 'info' or command == '-i':
            info()
        elif command == 'dep' or command == '-d':
            deploy()
        elif command == 'help' or command == '-h':
            help()
        elif command == 'all_models' or command == '-am':
            all_models(sys.argv)
        elif command == 'model' or command == '-m':
            one_model(sys.argv)
        elif command == 'all_apis' or command == '-aa':
            all_apis(sys.argv)
        elif command == 'api' or command == '-a':
            one_api(sys.argv)
        if command == 'all_views' or command == '-av':
            all_views(sys.argv)
        elif command == 'view' or command == '-v':
            one_view(sys.argv)
        elif command == 'all_controllers' or command == '-ac':
            all_controllers(sys.argv)
        elif command == 'controller' or command == '-c':
            one_controller(sys.argv)
        else:
            help()
    except Exception as e:
        print 'Error, use -h for help'
        print e.message

if __name__ == "__main__":
    main()


from . import config
import hashlib


class Edit:

    def GET(self, username):
        result = config.model.get_users(username)
        return config.render.edit(result)

    def POST(self, username):
        form = config.web.input()
        pwdhash = hashlib.md5(form.password).hexdigest()

        config.model.edit_users(
            form['username'],
            pwdhash,
            form['privilege']
        )
        raise config.web.seeother('/users')


import config

class Edit:
    
    def GET(self, primary_key):
        result = config.model.get_table_name(int(primary_key))
        return config.render.edit(result)

    def POST(self, primary_key):
        form = config.web.input()

        config.model.edit_table_name(
            fields
            
        )
        raise config.web.seeother('/table_name')


import config
import app


class Logout:
    def GET(self):
        app.session.username = None
        app.session.privilege = 0
        app.session.kill()
        raise config.web.seeother('/login')